; ************************************************
; *** 737 Demo - 1-Feb-2008 by Kevin Krausnick ***
; *** Ported to Acme Assembler on 14-Sep-2024  ***
; ************************************************

!addr GETIN      = $FFE4
!addr SCREENRAM  = $0400
!addr SID        = $D400
!addr TURBINEDAT = $3F40 ;16192 - TURBINE ENGINE SOURCE DATA (1120 BYTES)
!addr COLORMAP   = $1C00 ;7168 - BITMAP COLOR VALUES (1000 BYTES - LOADED WITH BITMAP)
!addr TARGET1    = $3548 ;BITMAP TARGET FOR TURBINE ENGINE ON LEFT SIDE OF SCREEN
!addr TARGET2    = $3640 ;BITMAP TARGET FOR TURBINE ENGINE ON RIGHT SIDE OF SCREEN
!addr CLR        = 42590 ;CLEAR VARIABLES AND RESET MEMORY POINTERS
!addr VICCTRL    = 53265 ;VIC-II VERTICAL SCROLL AND CONTROL REGISTER
!addr VICMEM     = 53272 ;VIC-II MEMORY CONTROL REGISTER
!addr BORDER     = 53280 ;SCREEN BORDER COLOR
!addr BACKGROUND = 53281 ;SCREEN BACKGROUND COLOR
!addr LETTERS    = $53   ;PRESSING "S" TOGGLES SOUND ON OR OFF
!addr WHITE      = 1     ;BORDER AND BACKGROUND COLOR
!addr CLS        = 147   ;CBM ASCII CODE FOR CLEAR SCREEN
!addr TIME       = 162   ;JIFFY CLOCK TIME REGISTER

* = $801
; basic header
!hex 24080a008f203733372044454d4f202d20312d4645422d32303038204259204b
!hex 504b004b0814008f2050524553532027532720544f20544f47474c4520534f55
!hex 4e44204f4e2f4f46460067081e008f20414e59204f54484552204b455920544f
!hex 204558495400850828009e31373331323a8f2052454e44455220414e494d4154
!hex 494f4e000000

* = $1c00 ; 7168
!src "737_dat.asm"

* = $43A0 ;17312 - SYS TO THIS ADDRESS TO RUN DEMO

;--------------------------------------------------------------------------------

; *** MAIN PROGRAM LOOP ***

    JSR INIT     ;INITIALIZE

M1  JSR TURBINE  ;MAIN LOOP OF ANIMATION
    JSR T2
    JSR T2
    JSR T2
    JSR GETIN
    BEQ M1
    CMP #LETTERS ;LETTER "S" TOGGLES SOUND ON OR OFF
    BNE M2
    INC SOUND
    JSR SOUNDON  ;TOGGLE SOUND
    JMP M1

M2  JSR CLEANUP  ;PREPARE TO EXIT

    RTS          ;EXIT

;--------------------------------------------------------------------------------

; *** INITIALIZATION ***

INIT = *

    LDA #28      ;RESERVE MEMORY ABOVE ADDRESS 7168 (7168 / 256 = 28)
    STA 52       ;RESERVE MEMORY FOR VARIABLES
    STA 56       ;RESERVE MEMORY FOR BASIC
    JSR CLR      ;CLEAR VARIABLES AND RESET MEMORY POINTERS

    LDA #WHITE
    STA BORDER
    STA BACKGROUND ;SET BORDER AND BACKGROUND TO WHITE

    LDA VICCTRL  ;53265 - VIC-II VERTICAL SCROLL AND CONTROL REGISTER
    STA OLDVICCTRL
    AND #255-16  ;BLANK THE SCREEN
    ORA #32      ;ENTER BIT MAP MODE
    STA VICCTRL

    LDA VICMEM   ;53272 - VIC-II MEMORY CONTROL REGISTER
    STA OLDVICMEM
    ORA #8       ;BIT MAP OFFSET IS 8K FROM VIC-II START ADDRESS
    STA VICMEM   ;SET VIC-II MEMORY CONTROL REGISTER

    JSR MAPCOLORS ;ASSIGN BIT MAP COLORS

    JSR INITSID  ;INITIALIZE SID CHIP
    JSR SOUNDON  ;TURN ON SOUND IF ENABLED

    LDA VICCTRL
    ORA #16      ;UNBLANK THE SCREEN
    STA VICCTRL
    RTS

;--------------------------------------------------------------------------------

; *** SAVE ORIGINAL SCREEN AND MAP COLOR VALUES TO BITMAP ***

MAPCOLORS = *

    LDY #0

L1  LDA SCREENRAM,Y
    STA SAVESCREEN,Y
    LDA COLORMAP,Y
    STA SCREENRAM,Y

    LDA SCREENRAM + $100,Y
    STA SAVESCREEN + $100,Y
    LDA COLORMAP + $100,Y
    STA SCREENRAM + $100,Y

    LDA SCREENRAM + $200,Y
    STA SAVESCREEN + $200,Y
    LDA COLORMAP + $200,Y
    STA SCREENRAM + $200,Y

    LDA SCREENRAM + $300,Y
    STA SAVESCREEN + $300,Y
    LDA COLORMAP + $300,Y
    STA SCREENRAM + $300,Y

    DEY
    BNE L1
    RTS

;--------------------------------------------------------------------------------

; *** RESTORE ORIGINAL SCREEN ***

RESTORESCRN = *

    LDY #0

R1  LDA SAVESCREEN,Y
    STA SCREENRAM,Y

    LDA SAVESCREEN + $100,Y
    STA SCREENRAM + $100,Y

    LDA SAVESCREEN + $200,Y
    STA SCREENRAM + $200,Y

    LDA SAVESCREEN + $300,Y
    STA SCREENRAM + $300,Y

    DEY
    BNE R1
    RTS

;--------------------------------------------------------------------------------

; *** TURN SOUND ON ***

SOUNDON = *

; MASTER VOLUME & FILTER

    LDA #16+15
    STA SID+24   ;MASTER VOLUME MAX, LOW-PASS FILTER
    LDA #3
    STA SID+22   ;FILTER FREQUENCY HIGH BYTE
    LDA #240+4
    STA SID+23   ;FILTER VOICE 3, APPLY MAXIMUM RESONANCE

; VOICE 1 - ENGINE

    LDA #100
    STA SID+1    ;VOICE 1 FREQUENCY HIGH BYTE
    LDA #16
    STA SID+6    ;VOICE 1 SUSTAIN VOLUME

; VOICE 2 - RING MODULATION

    LDA #200
    STA SID+7    ;VOICE 2 FREQUENCY LOW BYTE
    LDA #16
    STA SID+13   ;VOICE 2 SUSTAIN VOLUME

; VOICE 3 - LOW RUMBLE

    LDA #3
    STA SID+15   ;VOICE 3 FREQUENCY HIGH BYTE
    LDA #144
    STA SID+19   ;VOICE 3 ATTACK/DECAY
    LDA #249
    STA SID+20   ;VOICE 3 SUSTAIN RELEASE

; TRIGGER SOUNDS OR TURN THEM OFF

    LDA SOUND    ;PREPARE TO USE FLAG TO TURN SOUNDS ON OR OFF
    AND #1
    STA SOUND

    LDA #16
    ORA SOUND
    STA SID+4    ;VOICE 1 TRIANGLE WAVEFORM AND GATE BIT
    LDA #16+4
    ORA SOUND
    STA SID+11   ;VOICE 2 TRIANGLE WAVEFORM, RING MODULATION AND GATE BIT
    LDA #128
    ORA SOUND
    STA SID+18   ;VOICE 3 NOISE WAVEFORM AND GATE BIT

    RTS

;--------------------------------------------------------------------------------

; *** INITIALIZE SID CHIP OR TURN OFF SOUND ***

INITSID = *      ;INITIALIZE SID CHIP

    LDA #0
    TAX
S1  STA SID,X
    INX
    CPX #25
    BNE S1

    RTS

;--------------------------------------------------------------------------------

; *** PREPARE TO EXIT ***

CLEANUP = *

    JSR INITSID  ;TURN OFF SOUND

    LDA VICCTRL
    AND #255-16  ;BLANK THE SCREEN
    STA VICCTRL

    JSR RESTORESCRN ;RESTORE ORIGINAL SCREEN VALUES

    LDA OLDVICMEM
    STA VICMEM   ;RESTORE ORIGINAL BIT MAP OFFSET FROM VIC-II START ADDRESS

    LDA OLDVICCTRL
    STA VICCTRL  ;EXIT BIT MAP MODE AND UNBLANK THE SCREEN

    RTS

;--------------------------------------------------------------------------------

; *** COPY TURBINE ENGINE FRAME (4 FRAMES TOTAL IN ANIMATION) ***

TURBINE = *      ;FRAME 0

    LDA #<TURBINEDAT
    STA $F9
    LDA #>TURBINEDAT
    STA $FA      ;START OF TURBINE ENGINE FRAMES (SOURCE DATA)

T2 = *           ;FRAME 1-3

    LDA #<TARGET1
    STA $FB
    LDA #>TARGET1
    STA $FC      ;TARGET FOR TURBINE ENGINE ON LEFT SIDE OF SCREEN

    LDA #<TARGET2
    STA $FD
    LDA #>TARGET2
    STA $FE      ;TARGET FOR TURBINE ENGINE ON RIGHT SIDE OF SCREEN

    LDX #5       ;EACH FRAME CONSISTS OF 5 CONTIGUOUS ROWS OF DATA
L2  LDY #0
L3  LDA ($F9),Y
    STA ($FB),Y
    STA ($FD),Y  ;COPY SOURCE DATA TO SCREEN TARGET AREAS
    INY
    CPY #56      ;7 COLUMNS * 8 BYTES PER "CHAR" = 56 BYTES PER ROW TO COPY
    BNE L3

    CLC
    LDA $F9
    ADC #56
    STA $F9
    LDA $FA
    ADC #0
    STA $FA      ;SKIP 56 BYTES OF SOURCE DATA FOR NEXT "ROW"

    CLC
    LDA $FB
    ADC #64
    STA $FB
    LDA $FC
    ADC #1
    STA $FC      ;SKIP 320 BYTES OF TARGET DATA FOR NEXT SCREEN "ROW" (TURBINE 1)

    CLC
    LDA $FD
    ADC #64
    STA $FD
    LDA $FE
    ADC #1
    STA $FE      ;SKIP 320 BYTES OF TARGET DATA FOR NEXT SCREEN "ROW" (TURBINE 2)

    DEX
    BNE L2

    JSR STROBES  ;FLASH STROBES ON FUSELAGE
    JSR DELAY    ;PACE THE ANIMATION
    RTS

;--------------------------------------------------------------------------------

; *** PACE THE ANIMATION WITH A TIME DELAY BETWEEN FRAMES ***

DELAY = *

    LDA TIME
D1  CMP TIME
    BEQ D1
    RTS

;--------------------------------------------------------------------------------

; *** FLASH STROBES ON FUSELAGE ***

STROBES = *

    INC STROBE1
    LDA STROBE1
    CMP #96
    BCC L4
    LDA #0
    STA STROBE1
L4  LDX #$A1
    LDA STROBE1
    CMP #16
    BCC L5
    LDX #$C1
L5  STX SCREENRAM+299 ;TARGET COLOR BYTE FOR UPPER STROBE (LEFT)
    STX SCREENRAM+300 ;TARGET COLOR BYTE FOR UPPER STROBE (RIGHT)

    INC STROBE2
    LDA STROBE2
    CMP #96
    BCC L6
    LDA #0
    STA STROBE2
L6  LDX #$A1
    LDA STROBE2
    CMP #16
    BCC L7
    LDX #$C1
L7  STX SCREENRAM+899 ;TARGET COLOR BYTE FOR LOWER STROBE (LEFT)
    STX SCREENRAM+900 ;TARGET COLOR BYTE FOR LOWER STROBE (RIGHT)

    RTS

;--------------------------------------------------------------------------------

; *** PROGRAM VARIABLES ***

OLDVICCTRL !BYTE 0  ;ORIGINAL VIC-II CONTROL REGISTER VALUE TO RESTORE
OLDVICMEM  !BYTE 0  ;ORIGINAL VIC-II MEMORY CONTROL REGISTER VALUE TO RESTORE
SOUND      !BYTE 1  ;FLAG FOR TOGGLING SOUND ON OR OFF
STROBE1    !BYTE 0  ;TIMING CYCLE FOR UPPER STROBE LIGHT
STROBE2    !BYTE 48 ;TIMING CYCLE FOR LOWER STROBE LIGHT

SAVESCREEN = *      ;SAVED TEXT SCREEN (1000 BYTES)